// ------------------------------------------------------------------------------------------------

/*
    This file is part of Style Sheet Chooser.

    Style Sheet Chooser is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Style Sheet Chooser is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Style Sheet Chooser; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ------------------------------------------------------------------------------------------------
//
// Style Sheet Chooser, v0.01.
//
// Copyright 2004, Rob Tougher (http://www.robtougher.com)
//
// This extension persists your choices for alternate stylesheets on
// a per-domain basis. For example, if you browse to http://domain.com/index1.html,
// choose the alternate stylesheet "Alt", and browse to http://domain.com/index2.html,
// the alternate stylesheet "Alt" will be used on the second page, index2.html.
//
// Choices are persisted between window instances, so your choices will be remembered
// if you close Firefox and restart it.
//
// ------------------------------------------------------------------------------------------------






//
// Hold onto an instance of the observer service. We use this to pass
// selection cache events between windows.
//
var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(
					Components.interfaces.nsIObserverService);

//
// We use the directory service to find the user's profile directory.
//
var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].createInstance(
									Components.interfaces.nsIProperties);



// ------------------------------------------------------------------------------------------------
//
// Selection cache code.
//
// ------------------------------------------------------------------------------------------------


//
// Global cache of stylesheet selections.
//
var selectionCache = new Array();



function addToCache(key, value)
{
	selectionCache[key] = value;
	saveSelectionCache();
	observerService.notifyObservers(null, "StyleSheetChooser.SelectionCacheUpdated", "");
}
function getCacheValue(key)
{
	return selectionCache[key];
}
function keyExistsInCache(key)
{
	return typeof(selectionCache[key]) != "undefined";
}
function makeCacheKey(browserObject)
{
	try
	{
		return browserObject.contentDocument.location.hostname;
	}
	catch(e)
	{
		return "undefined";
	}
}
function getCacheFileName()
{
	var file = directoryService.get("ProfD", Components.interfaces.nsIFile);
	file.append('StyleSheetChooser-0.1.txt');
	return file.path;
}
function getCacheFile()
{
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	file.initWithPath(getCacheFileName());

	if(file.exists() == false)
	{
		file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420);
	}

	return file;
}
function loadSelectionCache()
{
	selectionCache = new Array();

	var fileInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(
									Components.interfaces.nsIFileInputStream);

	fileInputStream.init(getCacheFile(), 0x01, 00004, null);

	var scriptableInputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(
								Components.interfaces.nsIScriptableInputStream);

	scriptableInputStream.init(fileInputStream);

	var sData = scriptableInputStream.read(scriptableInputStream.available());
	var items = sData.split('\n');
	for(var i = 0; i < items.length - 1; i+=2)
	{
		var key = items[i];
		var value = items[i+1];
		selectionCache[key] = value;
	}

	fileInputStream.close();
}
function saveSelectionCache()
{
	var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(
									Components.interfaces.nsIFileOutputStream);
	fileOutputStream.init(getCacheFile(), 0x04 | 0x08 | 0x20, 420, 0);

	for(key in selectionCache)
	{
		var s = key + '\n';
		fileOutputStream.write(s, s.length);
		s = selectionCache[key] + '\n';
		fileOutputStream.write(s, s.length);
	}

	fileOutputStream.close();
}


function removeFromCache(sKeyToRemove)
{
	var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(
									Components.interfaces.nsIFileOutputStream);
	fileOutputStream.init(getCacheFile(), 0x04 | 0x08 | 0x20, 420, 0);

	for(key in selectionCache)
	{
		if(key != sKeyToRemove)
		{
			var s = key + '\n';
			fileOutputStream.write(s, s.length);
			s = selectionCache[key] + '\n';
			fileOutputStream.write(s, s.length);
		}
	}

	fileOutputStream.close();

	observerService.notifyObservers(null, "StyleSheetChooser.SelectionCacheUpdated", "");
}


//
// selectionCacheObserver
//
// We throw a "SelectionCacheUpdated" event when the user has chosen
// a theme. This allows all browsers to update their selection caches.
//
var selectionCacheObserver = {
	observe: function(subject, sTopic, sData)
	{
		if (sTopic == "StyleSheetChooser.SelectionCacheUpdated")
		{
			// Reload the selection cache file.
			loadSelectionCache();
		}
	}
};




function updateStyleSheetDisabledStates()
{
	try
	{
		var browsers = document.getElementById("content").browsers;
		for(var i = 0; i < browsers.length; i++)
		{
	//		if(browsers[i] == document.getElementById("content").mCurrentBrowser)
	//		{
				if(keyExistsInCache(makeCacheKey(browsers[i])))
				{
					switchStyleSheetInFrame(browsers[i].contentWindow,
							getCacheValue(makeCacheKey(browsers[i])));
				}
	//		}
	
		}
	}
	catch(e)
	{
		//
	}
}





var updateStyleSheetsDisabledStatesObserver = {
	observe: function(subject, sTopic, sData)
	{
		if (sTopic == "StyleSheetChooser.UpdateStyleSheetsDisabledStates")
		{
			updateStyleSheetDisabledStates();
		}
	}
};


function fireUpdateStyleSheetsDisabledStates()
{
	updateStyleSheetDisabledStates();
	observerService.notifyObservers(null, "StyleSheetChooser.UpdateStyleSheetsDisabledStates", "");
}




// ------------------------------------------------------------------------------------------------
//
// Progress listener code.
//
// ------------------------------------------------------------------------------------------------

function StyleSheetChooserProgressListener()
{
}

StyleSheetChooserProgressListener.prototype =
{
	onLocationChange : function(aWebProgress, aRequest, aLocation)
	{
		try
		{
			var currentBrowser = document.getElementById("content").selectedBrowser;
			var sLocation = currentBrowser.contentDocument.location;
			var sTitle = currentBrowser.contentDocument.title;
	
			if(sLocation != "about:blank")
			{
				document.getElementById("urlbar").value = sLocation;
				window.title = sTitle;
			}
	
			currentBrowser.contentWindow.focus();
		}
		catch(e)
		{
			// For mouse gestures error.
		}
	},

	onProgressChange : function(aWebProgress, aRequest, aCurSelfProgress,
				aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress)
	{
	},

	onSecurityChange : function(aWebProgress, aRequest, state) {},

	onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus)
	{
		try
		{
			updateStyleSheetDisabledStates();
		}
		catch(e)
		{
			// For mouse gestures error.
		}
	},

	onStatusChange : function(aWebProgress, aRequest, aStatus, message)
	{
		try
		{
			updateStyleSheetDisabledStates();
		}
		catch(e)
		{
			// For mouse gestures error.
		}
	},

	QueryInterface : function(aIID)
	{

		if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
			aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
			aIID.equals(Components.interfaces.nsIXULBrowserWindow) ||
			aIID.equals(Components.interfaces.nsISupports))
			return this;
		throw Components.results.NS_NOINTERFACE;
	},

	// See bug 123340 (http://bugzilla.mozilla.org/show_bug.cgi?id=123340)
	// This function is not part of the progress listener interface, but the
	// tab browser calls it anyway.
	onLinkIconAvailable : function(a, b)
	{
	}

}






// ------------------------------------------------------------------------------------------------
//
// Stylesheet switching code.
//
// ------------------------------------------------------------------------------------------------



//
// setStyleSheetDisabledProperty
//
function setStyleSheetDisabledProperty(styleSheet, isDisabled)
{
	if(styleSheet.disabled != isDisabled)
	{
		styleSheet.disabled = isDisabled;
	}
}



function isStyleSheetValidForFrame(frame, sSelection)
{
	if(sSelection == "_nostyle")
	{
		return true;
	}
	else if (sSelection == "_default")
	{
		return true;
	}
	else
	{
		for(var j = 0; j < frame.document.styleSheets.length; j++)
		{
			var styleSheet = frame.document.styleSheets[j];
			if(styleSheet.title == sSelection)
			{
				return true;
			}
		}
	}

	return false;
}

//
// switchStyleSheetInFrame
//
// Updates the disabled states of all styleSheets in this
// frame, and then recursively calls itself for all
// child frames.
//
//		"_nostyle" -- disabled everything
//		"valid title" -- enables preferred stylesheets, and stylesheets
//				matching this title.
//
function switchStyleSheetInFrame(frame, sSelection)
{
	if(isStyleSheetValidForFrame(frame, sSelection))
	{
		for(var j = 0; j < frame.document.styleSheets.length; j++)
		{
			var styleSheet = frame.document.styleSheets[j];
	
			// Make sure this is a "media=screen" style sheet.
			if(isValidMedia(styleSheet))
			{
				// Chose "None".
				if (sSelection == "_nostyle")
				{
					setStyleSheetDisabledProperty(styleSheet, true);
				}
				// Chose default.
				else if(sSelection == "_default")
				{
					if(isPersistent(styleSheet) || isPreferred(styleSheet))
					{
						setStyleSheetDisabledProperty(styleSheet, false);
					}
					else
					{
						setStyleSheetDisabledProperty(styleSheet, true);
					}
				}
				// Chose style sheet.
				else
				{
					if(isPersistent(styleSheet))
					{
						setStyleSheetDisabledProperty(styleSheet, false);
					}
					else
					{
						setStyleSheetDisabledProperty(styleSheet, (styleSheet.title != sSelection));
					}
				}
			}
		}
	}


	// Next, loop through the child frames.
	for(var i = 0; i < frame.frames.length; i++)
	{
		switchStyleSheetInFrame(frame.frames[i], sSelection);
	}
}





//
// themeSelected
//
// This event handler is called when the user selects an item from the
// alternate stylesheet right-click context menu.
//
//
function themeSelected(evt)
{
	// Cache the user's selection.
	addToCache(makeCacheKey(document.getElementById("content").mCurrentBrowser), evt.target.getAttribute('data'));
	fireUpdateStyleSheetsDisabledStates();
}



//
// Event handler for the window's load event. We initialize
// ourselves here.
//
function windowLoad()
{
	var pageThemeButton = document.getElementById("page-theme-button");


	// Load the selection cache from the file system.
	loadSelectionCache();

	//
	// We want to receive "command" events from the alternate stylesheet menu.
	//
	// var menuPopup = pageThemeButton.firstChild;
	// menuPopup.addEventListener("command", themeSelected, false);


	// Listen to our own topic, "StyleSheetChooser.SelectionCacheUpdated". This allows us
	// to keep the selection cache synchronized between browser windows.
	observerService.addObserver(selectionCacheObserver, "StyleSheetChooser.SelectionCacheUpdated", false);


	observerService.addObserver(updateStyleSheetsDisabledStatesObserver,
					"StyleSheetChooser.UpdateStyleSheetsDisabledStates", false);


	// Keep one progress listener per browser. Any time a state or location
	// change happens, update the stylesheet disabled states for all tabs
	// in all windows.
	var listener = new StyleSheetChooserProgressListener();
	document.getElementById("content").addProgressListener(
			listener.QueryInterface(Components.interfaces.nsIWebProgressListener), 
			Components.interfaces.nsIWebProgress.NOTIFY_ALL);

}




function styleSheetChooserMenuClicked(evt)
{
	var sTitle = this.getAttribute("id");

	// Cache the user's selection.
	addToCache(makeCacheKey(document.getElementById("content").mCurrentBrowser), sTitle);
	fireUpdateStyleSheetsDisabledStates();

}



//
// Stylesheet type helper functions.
//

function isAlternate(styleSheet)
{
	// Not a 'link' element?
	if(styleSheet.ownerNode == null || styleSheet.ownerNode.nodeName.toLowerCase() != 'link')
	{
		return false;
	}

	if(styleSheet.ownerNode.getAttribute('rel') == 'alternate stylesheet')
	{
		return true;
	}
	else
	{
		return false;
	}
}

function isPreferred(styleSheet)
{
	return ! isAlternate(styleSheet) && styleSheet.title != null && styleSheet.title != '';
}

function isPersistent(styleSheet)
{
	return ! isAlternate(styleSheet) && (styleSheet.title == null || styleSheet.title == '');
}

function isValidMedia(styleSheet)
{
	var sMedia = styleSheet.media.mediaText.toLowerCase();


	if(sMedia == null || sMedia == '' || sMedia.indexOf("all") != -1 || sMedia.indexOf("screen") != -1)
	{
		return true;
	}
	else
	{
		return false;
	}
}



//
// Create the "Use Style" menu, so that the user can choose one of the style sheets.
//
function buildStyleSheetChooserMenu(useStyleMenuPopup)
{

	while(useStyleMenuPopup.firstChild)
	{
		useStyleMenuPopup.removeChild(useStyleMenuPopup.firstChild);
	}


	var currentBrowser = document.getElementById("content").selectedBrowser;
	var currentDocument = currentBrowser.contentDocument;
	var useStyleMenu = document.getElementById("use-style-menu");
	var bHasStyleSheet = false;


	// None
	var menuItem = document.createElement("menuitem");
	menuItem.setAttribute("id", "_nostyle");
	menuItem.setAttribute("label", "None");
	useStyleMenuPopup.appendChild(menuItem);
	menuItem.addEventListener("command",
				styleSheetChooserMenuClicked,
				true);


	// Default
	menuItem = document.createElement("menuitem");
	menuItem.setAttribute("id", "_default");
	menuItem.setAttribute("label", "Default");
	useStyleMenuPopup.appendChild(menuItem);
	menuItem.addEventListener("command",
				styleSheetChooserMenuClicked,
				true);



	var processedStyleSheets = new Array();

	for(var i = 0; i < currentDocument.styleSheets.length; i++)
	{
		var styleSheet = currentDocument.styleSheets[i];

		if(isValidMedia(styleSheet) && ! isPersistent(styleSheet)
			&& processedStyleSheets[styleSheet.title] != styleSheet.title)
		{
			if(! bHasStyleSheet)
			{
				menuItem = document.createElement("menuseparator");
				menuItem.setAttribute("id", "__separator");
				useStyleMenuPopup.appendChild(menuItem);
				bHasStyleSheet = true;
			}

			menuItem = document.createElement("menuitem");
			menuItem.setAttribute("id", styleSheet.title);
			menuItem.setAttribute("label", styleSheet.title);
			useStyleMenuPopup.appendChild(menuItem);
			menuItem.addEventListener("command",
						styleSheetChooserMenuClicked,
						true);
			
			processedStyleSheets[styleSheet.title] = styleSheet.title;
		}
	}
}





// ------------------------------------------------------------------------------------------------
//
// Main initialization. This line of code gets run when the extension
// is loaded. It adds an event listener for the window load event -- see
// the "windowLoad" function for this extension's initialization code.
//
// ------------------------------------------------------------------------------------------------

window.addEventListener("load", windowLoad, false);




